@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
 Exceptions for pyproj
 """
@@ -9,7 +8,7 @@ class ProjError(RuntimeError):
 
     internal_proj_error = None
 
-    def __init__(self, error_message):
+    def __init__(self, error_message: str) -> None:
         if self.internal_proj_error is not None:
             error_message = (
                 "{error_message}: (Internal Proj Error: {internal_proj_error})"
@@ -17,8 +16,15 @@ class ProjError(RuntimeError):
                 error_message=error_message,
                 internal_proj_error=self.internal_proj_error,
             )
-            self.internal_proj_error = None
-        super(ProjError, self).__init__(error_message)
+            ProjError.clear()
+        super().__init__(error_message)
+
+    @staticmethod
+    def clear() -> None:
+        """
+        This will clear the internal PROJ erro message.
+        """
+        ProjError.internal_proj_error = None
 
 
 class CRSError(ProjError):
