@@ -1,48 +1,135 @@
-# -*- coding: utf-8 -*-
 """
 The transformer module is for performing cartographic transformations.
+"""
 
-Copyright (c) 2019 pyproj Contributors.
-
-Permission to use, copy, modify, and distribute this software
-and its documentation for any purpose and without fee is hereby
-granted, provided that the above copyright notice appear in all
-copies and that both the copyright notice and this permission
-notice appear in supporting documentation. THE AUTHOR DISCLAIMS
-ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
-IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT
-SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR
-CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
-LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
-NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
-CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE."""
-
+__all__ = [
+    "transform",
+    "itransform",
+    "Transformer",
+    "TransformerGroup",
+    "AreaOfInterest",
+]
+import warnings
 from array import array
 from itertools import chain, islice
+from typing import Any, Iterable, Iterator, List, Optional, Tuple, Union
 
 from pyproj import CRS, Proj
-from pyproj._transformer import _Transformer
+from pyproj._crs import AreaOfUse, CoordinateOperation
+from pyproj._transformer import AreaOfInterest, _Transformer, _TransformerGroup  # noqa
 from pyproj.compat import cstrencode
 from pyproj.enums import TransformDirection, WktVersion
 from pyproj.exceptions import ProjError
 from pyproj.utils import _convertback, _copytobuffer
 
-try:
-    from future_builtins import zip  # python 2.6+
-except ImportError:
-    pass  # python 3.x
+
+class TransformerGroup(_TransformerGroup):
+    """
+    The TransformerGroup is a set of possible transformers from one CRS to another.
+
+    .. versionadded:: 2.3.0
+
+    From PROJ docs::
+
+        The operations are sorted with the most relevant ones first: by
+        descending area (intersection of the transformation area with the
+        area of interest, or intersection of the transformation with the
+        area of use of the CRS), and by increasing accuracy. Operations
+        with unknown accuracy are sorted last, whatever their area.
+
+    """
+
+    def __init__(
+        self,
+        crs_from: Any,
+        crs_to: Any,
+        skip_equivalent: bool = False,
+        always_xy: bool = False,
+        area_of_interest: Optional[AreaOfInterest] = None,
+    ) -> None:
+        """Get all possible transformations from a :obj:`pyproj.crs.CRS`
+        or input used to create one.
 
 
-class Transformer(object):
+        Parameters
+        ----------
+        crs_from: pyproj.crs.CRS or input used to create one
+            Projection of input data.
+        crs_to: pyproj.crs.CRS or input used to create one
+            Projection of output data.
+        skip_equivalent: bool, optional
+            If true, will skip the transformation operation if input and output
+            projections are equivalent. Default is false.
+        always_xy: bool, optional
+            If true, the transform method will accept as input and return as output
+            coordinates using the traditional GIS order, that is longitude, latitude
+            for geographic CRS and easting, northing for most projected CRS.
+            Default is false.
+        area_of_interest: :class:`pyproj.transformer.AreaOfInterest`, optional
+            The area of interest to help order the transformations based on the
+            best operation for the area.
+
+        """
+        super().__init__(
+            CRS.from_user_input(crs_from),
+            CRS.from_user_input(crs_to),
+            skip_equivalent=skip_equivalent,
+            always_xy=always_xy,
+            area_of_interest=area_of_interest,
+        )
+        for iii, transformer in enumerate(self._transformers):
+            self._transformers[iii] = Transformer(transformer)
+
+    @property
+    def transformers(self) -> List["Transformer"]:
+        """
+        list[:obj:`Transformer`]:
+            List of available :obj:`Transformer`
+            associated with the transformation.
+        """
+        return self._transformers
+
+    @property
+    def unavailable_operations(self) -> List[CoordinateOperation]:
+        """
+        list[:obj:`pyproj.crs.CoordinateOperation`]:
+            List of :obj:`pyproj.crs.CoordinateOperation` that are not
+            available due to missing grids.
+        """
+        return self._unavailable_operations
+
+    @property
+    def best_available(self) -> bool:
+        """
+        bool: If True, the best possible transformer is available.
+        """
+        return self._best_available
+
+    def __repr__(self) -> str:
+        return (
+            "<TransformerGroup: best_available={best_available}>\n"
+            "- transformers: {transformers}\n"
+            "- unavailable_operations: {unavailable_operations}"
+        ).format(
+            best_available=self.best_available,
+            transformers=len(self.transformers),
+            unavailable_operations=len(self.unavailable_operations),
+        )
+
+
+class Transformer:
     """
     The Transformer class is for facilitating re-using
     transforms without needing to re-create them. The goal
     is to make repeated transforms faster.
 
     Additionally, it provides multiple methods for initialization.
+
+    .. versionadded:: 2.1.0
+
     """
 
-    def __init__(self, base_transformer=None):
+    def __init__(self, base_transformer: Optional[_Transformer] = None) -> None:
         if not isinstance(base_transformer, _Transformer):
             ProjError.clear()
             raise ProjError(
@@ -52,49 +139,107 @@ class Transformer(object):
         self._transformer = base_transformer
 
     @property
-    def name(self):
+    def name(self) -> str:
         """
         str: Name of the projection.
         """
         return self._transformer.id
 
     @property
-    def description(self):
+    def description(self) -> str:
         """
         str: Description of the projection.
         """
         return self._transformer.description
 
     @property
-    def definition(self):
+    def definition(self) -> str:
         """
         str: Definition of the projection.
         """
         return self._transformer.definition
 
     @property
-    def has_inverse(self):
+    def has_inverse(self) -> bool:
         """
         bool: True if an inverse mapping exists.
         """
         return self._transformer.has_inverse
 
     @property
-    def accuracy(self):
+    def accuracy(self) -> float:
         """
         float: Expected accuracy of the transformation. -1 if unknown.
         """
         return self._transformer.accuracy
 
+    @property
+    def area_of_use(self) -> AreaOfUse:
+        """
+        .. versionadded:: 2.3.0
+
+        Returns
+        -------
+        AreaOfUse:
+            The area of use object with associated attributes.
+        """
+        return self._transformer.area_of_use
+
+    @property
+    def remarks(self) -> str:
+        """
+        .. versionadded:: 2.4.0
+
+        Returns
+        -------
+        str:
+            Remarks about object.
+        """
+        return self._transformer.remarks
+
+    @property
+    def scope(self) -> str:
+        """
+        .. versionadded:: 2.4.0
+
+        Returns
+        -------
+        str:
+            Scope of object.
+        """
+        return self._transformer.scope
+
+    @property
+    def operations(self) -> Optional[Tuple[CoordinateOperation]]:
+        """
+        .. versionadded:: 2.4.0
+
+        Returns
+        -------
+        Tuple[CoordinateOperation]:
+            The operations in a concatenated operation.
+        """
+        return self._transformer.operations
+
     @staticmethod
-    def from_proj(proj_from, proj_to, skip_equivalent=False, always_xy=False):
-        """Make a Transformer from a :obj:`~pyproj.proj.Proj` or input used to create one.
+    def from_proj(
+        proj_from: Any,
+        proj_to: Any,
+        skip_equivalent: bool = False,
+        always_xy: bool = False,
+        area_of_interest: Optional[AreaOfInterest] = None,
+    ) -> "Transformer":
+        """Make a Transformer from a :obj:`pyproj.proj.Proj` or input used to create one.
+
+        .. versionadded:: 2.1.2 skip_equivalent
+        .. versionadded:: 2.2.0 always_xy
+        .. versionadded:: 2.3.0 area_of_interest
 
         Parameters
         ----------
-        proj_from: :obj:`~pyproj.proj.Proj` or input used to create one
+        proj_from: :obj:`pyproj.proj.Proj` or input used to create one
             Projection of input data.
-        proj_to: :obj:`~pyproj.proj.Proj` or input used to create one
+        proj_to: :obj:`pyproj.proj.Proj` or input used to create one
             Projection of output data.
         skip_equivalent: bool, optional
             If true, will skip the transformation operation if input and output
@@ -104,10 +249,12 @@ class Transformer(object):
             coordinates using the traditional GIS order, that is longitude, latitude
             for geographic CRS and easting, northing for most projected CRS.
             Default is false.
+        area_of_interest: :class:`pyproj.transformer.AreaOfInterest`, optional
+            The area of interest to help select the transformation.
 
         Returns
         -------
-        :obj:`~Transformer`
+        Transformer
 
         """
         if not isinstance(proj_from, Proj):
@@ -115,24 +262,33 @@ class Transformer(object):
         if not isinstance(proj_to, Proj):
             proj_to = Proj(proj_to)
 
-        return Transformer(
-            _Transformer.from_crs(
-                proj_from.crs,
-                proj_to.crs,
-                skip_equivalent=skip_equivalent,
-                always_xy=always_xy,
-            )
+        return Transformer.from_crs(
+            proj_from.crs,
+            proj_to.crs,
+            skip_equivalent=skip_equivalent,
+            always_xy=always_xy,
+            area_of_interest=area_of_interest,
         )
 
     @staticmethod
-    def from_crs(crs_from, crs_to, skip_equivalent=False, always_xy=False):
-        """Make a Transformer from a :obj:`~pyproj.crs.CRS` or input used to create one.
+    def from_crs(
+        crs_from: Any,
+        crs_to: Any,
+        skip_equivalent: bool = False,
+        always_xy: bool = False,
+        area_of_interest: Optional[AreaOfInterest] = None,
+    ) -> "Transformer":
+        """Make a Transformer from a :obj:`pyproj.crs.CRS` or input used to create one.
+
+        .. versionadded:: 2.1.2 skip_equivalent
+        .. versionadded:: 2.2.0 always_xy
+        .. versionadded:: 2.3.0 area_of_interest
 
         Parameters
         ----------
-        crs_from: ~pyproj.crs.CRS or input used to create one
+        crs_from: pyproj.crs.CRS or input used to create one
             Projection of input data.
-        crs_to: ~pyproj.crs.CRS or input used to create one
+        crs_to: pyproj.crs.CRS or input used to create one
             Projection of output data.
         skip_equivalent: bool, optional
             If true, will skip the transformation operation if input and output
@@ -142,24 +298,26 @@ class Transformer(object):
             coordinates using the traditional GIS order, that is longitude, latitude
             for geographic CRS and easting, northing for most projected CRS.
             Default is false.
+        area_of_interest: :class:`pyproj.transformer.AreaOfInterest`, optional
+            The area of interest to help select the transformation.
 
         Returns
         -------
-        :obj:`~Transformer`
+        Transformer
 
         """
-        transformer = Transformer(
+        return Transformer(
             _Transformer.from_crs(
                 CRS.from_user_input(crs_from),
                 CRS.from_user_input(crs_to),
                 skip_equivalent=skip_equivalent,
                 always_xy=always_xy,
+                area_of_interest=area_of_interest,
             )
         )
-        return transformer
 
     @staticmethod
-    def from_pipeline(proj_pipeline):
+    def from_pipeline(proj_pipeline: str) -> "Transformer":
         """Make a Transformer from a PROJ pipeline string.
 
         https://proj.org/operations/pipeline.html
@@ -171,24 +329,27 @@ class Transformer(object):
 
         Returns
         -------
-        ~Transformer
+        Transformer
 
         """
         return Transformer(_Transformer.from_pipeline(cstrencode(proj_pipeline)))
 
     def transform(
         self,
-        xx,
-        yy,
-        zz=None,
-        tt=None,
-        radians=False,
-        errcheck=False,
-        direction=TransformDirection.FORWARD,
-    ):
+        xx: Any,
+        yy: Any,
+        zz: Any = None,
+        tt: Any = None,
+        radians: bool = False,
+        errcheck: bool = False,
+        direction: Union[TransformDirection, str] = TransformDirection.FORWARD,
+    ) -> Any:
         """
         Transform points between two coordinate systems.
 
+        .. versionadded:: 2.1.1 errcheck
+        .. versionadded:: 2.2.0 direction
+
         Parameters
         ----------
         xx: scalar or array (numpy or python)
@@ -202,14 +363,14 @@ class Transformer(object):
         radians: boolean, optional
             If True, will expect input data to be in radians and will return radians
             if the projection is geographic. Default is False (degrees). Ignored for
-            pipeline transformations.
-        errcheck: boolean, optional (default False)
+            pipeline transformations with pyproj 2, but will work in pyproj 3.
+        errcheck: boolean, optional
             If True an exception is raised if the transformation is invalid.
             By default errcheck=False and an invalid transformation
             returns ``inf`` and no exception is raised.
-        direction: ~pyproj.enums.TransformDirection, optional
+        direction: pyproj.enums.TransformDirection, optional
             The direction of the transform.
-            Default is :attr:`~pyproj.enums.TransformDirection.FORWARD`.
+            Default is :attr:`pyproj.enums.TransformDirection.FORWARD`.
 
 
         Example:
@@ -226,10 +387,11 @@ class Transformer(object):
         >>> pipe_trans = Transformer.from_pipeline(pipeline_str)
         >>> xt, yt = pipe_trans.transform(2.1, 0.001)
         >>> "%.3f  %.3f" % (xt, yt)
-        '120.321  0.057'
+        '2.100  0.001'
         >>> transproj = Transformer.from_crs(
         ...     {"proj":'geocent', "ellps":'WGS84', "datum":'WGS84'},
-        ...     '+init=EPSG:4326'
+        ...     "EPSG:4326",
+        ...     always_xy=True,
         ... )
         >>> xpj, ypj, zpj = transproj.transform(
         ...     -2704026.010,
@@ -240,8 +402,9 @@ class Transformer(object):
         >>> "%.3f %.3f %.3f" % (xpj, ypj, zpj)
         '-2.137 0.661 -20.531'
         >>> transprojr = Transformer.from_crs(
-        ...     '+init=EPSG:4326',
+        ...     "EPSG:4326",
         ...     {"proj":'geocent', "ellps":'WGS84', "datum":'WGS84'},
+        ...     always_xy=True,
         ... )
         >>> xpjr, ypjr, zpjr = transprojr.transform(xpj, ypj, zpj, radians=True)
         >>> "%.3f %.3f %.3f" % (xpjr, ypjr, zpjr)
@@ -278,23 +441,29 @@ class Transformer(object):
         outy = _convertback(yisfloat, yislist, xistuple, iny)
         return_data = (outx, outy)
         if inz is not None:
-            return_data += (_convertback(zisfloat, zislist, zistuple, inz),)
+            return_data += (  # type: ignore
+                _convertback(zisfloat, zislist, zistuple, inz),
+            )
         if intime is not None:
-            return_data += (_convertback(tisfloat, tislist, tistuple, intime),)
+            return_data += (  # type: ignore
+                _convertback(tisfloat, tislist, tistuple, intime),
+            )
         return return_data
 
     def itransform(
         self,
-        points,
-        switch=False,
-        time_3rd=False,
-        radians=False,
-        errcheck=False,
-        direction=TransformDirection.FORWARD,
-    ):
+        points: Any,
+        switch: bool = False,
+        time_3rd: bool = False,
+        radians: bool = False,
+        errcheck: bool = False,
+        direction: Union[TransformDirection, str] = TransformDirection.FORWARD,
+    ) -> Iterator[Iterable]:
         """
         Iterator/generator version of the function pyproj.Transformer.transform.
 
+        .. versionadded:: 2.1.1 errcheck
+        .. versionadded:: 2.2.0 direction
 
         Parameters
         ----------
@@ -309,13 +478,13 @@ class Transformer(object):
             If True, will expect input data to be in radians and will return radians
             if the projection is geographic. Default is False (degrees). Ignored for
             pipeline transformations.
-        errcheck: boolean, optional (default False)
+        errcheck: boolean, optional
             If True an exception is raised if the transformation is invalid.
             By default errcheck=False and an invalid transformation
             returns ``inf`` and no exception is raised.
-        direction: ~pyproj.enums.TransformDirection, optional
+        direction: pyproj.enums.TransformDirection, optional
             The direction of the transform.
-            Default is :attr:`~pyproj.enums.TransformDirection.FORWARD`.
+            Default is :attr:`pyproj.enums.TransformDirection.FORWARD`.
 
 
         Example:
@@ -334,10 +503,11 @@ class Transformer(object):
         >>> pipe_trans = Transformer.from_pipeline(pipeline_str)
         >>> for pt in pipe_trans.itransform([(2.1, 0.001)]):
         ...     '{:.3f} {:.3f}'.format(*pt)
-        '120.321 0.057'
+        '2.100 0.001'
         >>> transproj = Transformer.from_crs(
         ...     {"proj":'geocent', "ellps":'WGS84', "datum":'WGS84'},
-        ...     '+init=EPSG:4326',
+        ...     "EPSG:4326",
+        ...     always_xy=True,
         ... )
         >>> for pt in transproj.itransform(
         ...     [(-2704026.010, -4253051.810, 3895878.820)],
@@ -345,9 +515,10 @@ class Transformer(object):
         ... ):
         ...     '{:.3f} {:.3f} {:.3f}'.format(*pt)
         '-2.137 0.661 -20.531'
-        >>> transprojr = Transformer.from_proj(
-        ...     '+init=EPSG:4326',
+        >>> transprojr = Transformer.from_crs(
+        ...     "EPSG:4326",
         ...     {"proj":'geocent', "ellps":'WGS84', "datum":'WGS84'},
+        ...     always_xy=True,
         ... )
         >>> for pt in transprojr.itransform(
         ...     [(-2.137, 0.661, -20.531)],
@@ -356,8 +527,9 @@ class Transformer(object):
         ...     '{:.3f} {:.3f} {:.3f}'.format(*pt)
         '-2704214.394 -4254414.478 3894270.731'
         >>> transproj_eq = Transformer.from_proj(
-        ...     '+init=EPSG:4326',
+        ...     'EPSG:4326',
         ...     '+proj=longlat +datum=WGS84 +no_defs +type=crs',
+        ...     always_xy=True,
         ...     skip_equivalent=True
         ... )
         >>> for pt in transproj_eq.itransform([(-2.137, 0.661)]):
@@ -403,57 +575,109 @@ class Transformer(object):
             for pt in zip(*([iter(buff)] * stride)):
                 yield pt
 
-    def to_wkt(self, version=WktVersion.WKT2_2018, pretty=False):
+    def to_wkt(
+        self,
+        version: Union[WktVersion, str] = WktVersion.WKT2_2019,
+        pretty: bool = False,
+    ):
         """
         Convert the projection to a WKT string.
 
         Version options:
           - WKT2_2015
           - WKT2_2015_SIMPLIFIED
-          - WKT2_2018
-          - WKT2_2018_SIMPLIFIED
+          - WKT2_2019
+          - WKT2_2019_SIMPLIFIED
           - WKT1_GDAL
           - WKT1_ESRI
 
 
         Parameters
         ----------
-        version: ~pyproj.enums.WktVersion
+        version: pyproj.enums.WktVersion
             The version of the WKT output.
-            Default is :attr:`~pyproj.enums.WktVersion.WKT2_2018`.
+            Default is :attr:`pyproj.enums.WktVersion.WKT2_2019`.
         pretty: bool
             If True, it will set the output to be a multiline string. Defaults to False.
 
         Returns
         -------
-        str: The WKT string.
+        str:
+            The WKT string.
         """
         return self._transformer.to_wkt(version=version, pretty=pretty)
 
-    def __str__(self):
+    def to_json(self, pretty: bool = False, indentation: int = 2) -> str:
+        """
+        Convert the projection to a JSON string.
+
+        .. versionadded:: 2.4.0
+
+        Parameters
+        ----------
+        pretty: bool
+            If True, it will set the output to be a multiline string. Defaults to False.
+        indentation: int
+            If pretty is True, it will set the width of the indentation. Default is 2.
+
+        Returns
+        -------
+        str:
+            The JSON string.
+        """
+        return self._transformer.to_json(pretty=pretty, indentation=indentation)
+
+    def to_json_dict(self) -> dict:
+        """
+        Convert the projection to a JSON dictionary.
+
+        .. versionadded:: 2.4.0
+
+        Returns
+        -------
+        dict:
+            The JSON dictionary.
+        """
+        return self._transformer.to_json_dict()
+
+    def __str__(self) -> str:
         return self.definition
 
-    def __repr__(self):
-        return ("<{type_name}: {name}>\n" "{description}").format(
+    def __repr__(self) -> str:
+        return (
+            "<{type_name}: {name}>\nDescription: {description}\n"
+            "Area of Use:\n{area_of_use}"
+        ).format(
             type_name=self._transformer.type_name,
             name=self.name,
             description=self.description,
+            area_of_use=self.area_of_use or "- undefined",
         )
 
+    def __eq__(self, other: Any) -> bool:
+        if not isinstance(other, Transformer):
+            return False
+        return self._transformer.__eq__(other._transformer)
+
 
 def transform(
-    p1,
-    p2,
-    x,
-    y,
-    z=None,
-    tt=None,
-    radians=False,
-    errcheck=False,
-    skip_equivalent=False,
-    always_xy=False,
+    p1: Any,
+    p2: Any,
+    x: Any,
+    y: Any,
+    z: Any = None,
+    tt: Any = None,
+    radians: bool = False,
+    errcheck: bool = False,
+    skip_equivalent: bool = False,
+    always_xy: bool = False,
 ):
     """
+    .. versionadded:: 2.1.2 skip_equivalent
+    .. versionadded:: 2.2.0 always_xy
+
+    .. warning:: This function is deprecated. See: :ref:`upgrade_transformer`
+
     x2, y2, z2 = transform(p1, p2, x1, y1, z1)
 
     Transform points between two coordinate systems defined by the
@@ -494,9 +718,9 @@ def transform(
     >>> from pyproj import Proj, transform
     >>> # projection 1: UTM zone 15, grs80 ellipse, NAD83 datum
     >>> # (defined by epsg code 26915)
-    >>> p1 = Proj(init='epsg:26915', preserve_units=False)
+    >>> p1 = Proj('epsg:26915', preserve_units=False)
     >>> # projection 2: UTM zone 15, clrk66 ellipse, NAD27 datum
-    >>> p2 = Proj(init='epsg:26715', preserve_units=False)
+    >>> p2 = Proj('epsg:26715', preserve_units=False)
     >>> # find x,y of Jefferson City, MO.
     >>> x1, y1 = p1(-92.199881,38.56694)
     >>> # transform this point to projection 2 coordinates.
@@ -522,41 +746,38 @@ def transform(
     >>> xy = lons+lats
     >>> '%8.3f %8.3f %8.3f %5.3f %5.3f %5.3f' % xy
     ' -92.220  -94.720  -90.370 38.830 39.320 38.750'
-    >>> # test datum shifting, installation of extra datum grid files.
-    >>> p1 = Proj(proj='latlong',datum='WGS84')
-    >>> x1 = -111.5; y1 = 45.25919444444
-    >>> p2 = Proj(proj="utm",zone=10,datum='NAD27', preserve_units=False)
-    >>> x2, y2 = transform(p1, p2, x1, y1)
-    >>> "%s  %s" % (str(x2)[:9],str(y2)[:9])
-    '1402291.0  5076289.5'
-    >>> from pyproj import CRS
-    >>> c1 = CRS(proj='latlong',datum='WGS84')
-    >>> x1 = -111.5; y1 = 45.25919444444
-    >>> c2 = CRS(proj="utm",zone=10,datum='NAD27')
-    >>> x2, y2 = transform(c1, c2, x1, y1)
-    >>> "%s  %s" % (str(x2)[:9],str(y2)[:9])
-    '1402291.0  5076289.5'
-    >>> xeq, yeq = transform(4326, 4326, 30, 60, skip_equivalent=True)
-    >>> "%.0f %.0f" % (xeq, yeq)
-    '30 60'
     """
+    warnings.warn(
+        (
+            "This function is deprecated. "
+            "See: https://pyproj4.github.io/pyproj/stable/"
+            "gotchas.html#upgrading-to-pyproj-2-from-pyproj-1"
+        ),
+        DeprecationWarning,
+        stacklevel=2,
+    )
     return Transformer.from_proj(
         p1, p2, skip_equivalent=skip_equivalent, always_xy=always_xy
     ).transform(xx=x, yy=y, zz=z, tt=tt, radians=radians, errcheck=errcheck)
 
 
 def itransform(
-    p1,
-    p2,
-    points,
-    switch=False,
-    time_3rd=False,
-    radians=False,
-    errcheck=False,
-    skip_equivalent=False,
-    always_xy=False,
+    p1: Any,
+    p2: Any,
+    points: Iterable[Iterable],
+    switch: bool = False,
+    time_3rd: bool = False,
+    radians: bool = False,
+    errcheck: bool = False,
+    skip_equivalent: bool = False,
+    always_xy: bool = False,
 ):
     """
+    .. versionadded:: 2.1.2 skip_equivalent
+    .. versionadded:: 2.2.0 always_xy
+
+    .. warning:: This function is deprecated. See: :ref:`upgrade_transformer`
+
     points2 = itransform(p1, p2, points1)
     Iterator/generator version of the function pyproj.transform.
     Transform points between two coordinate systems defined by the
@@ -595,13 +816,13 @@ def itransform(
     >>> from pyproj import Proj, itransform
     >>> # projection 1: WGS84
     >>> # (defined by epsg code 4326)
-    >>> p1 = Proj(init='epsg:4326', preserve_units=False)
+    >>> p1 = Proj('epsg:4326', preserve_units=False)
     >>> # projection 2: GGRS87 / Greek Grid
-    >>> p2 = Proj(init='epsg:2100', preserve_units=False)
+    >>> p2 = Proj('epsg:2100', preserve_units=False)
     >>> # Three points with coordinates lon, lat in p1
     >>> points = [(22.95, 40.63), (22.81, 40.53), (23.51, 40.86)]
     >>> # transform this point to projection 2 coordinates.
-    >>> for pt in itransform(p1,p2,points): '%6.3f %7.3f' % pt
+    >>> for pt in itransform(p1,p2,points, always_xy=True): '%6.3f %7.3f' % pt
     '411050.470 4497928.574'
     '399060.236 4486978.710'
     '458553.243 4523045.485'
@@ -610,6 +831,15 @@ def itransform(
     '30 60'
 
     """
+    warnings.warn(
+        (
+            "This function is deprecated. "
+            "See: https://pyproj4.github.io/pyproj/stable/"
+            "gotchas.html#upgrading-to-pyproj-2-from-pyproj-1"
+        ),
+        DeprecationWarning,
+        stacklevel=2,
+    )
     return Transformer.from_proj(
         p1, p2, skip_equivalent=skip_equivalent, always_xy=always_xy
     ).itransform(
